.bk-root {
  @font-awesome: "./node_modules/font-awesome";

  @import "@{font-awesome}/less/variables.less";
  @fa-css-prefix: bk-u-fa;
  @import "@{font-awesome}/less/mixins.less";

  @font-face {
    font-family: 'FontAwesome';
    src: data-uri('@{font-awesome}/fonts/fontawesome-webfont.eot')   format('embedded-opentype'),
         data-uri('@{font-awesome}/fonts/fontawesome-webfont.woff2') format('woff2'),
         data-uri('@{font-awesome}/fonts/fontawesome-webfont.woff')  format('woff'),
         data-uri('@{font-awesome}/fonts/fontawesome-webfont.ttf')   format('truetype'),
         data-uri('@{font-awesome}/fonts/fontawesome-webfont.svg')   format('svg');
    font-weight: normal;
    font-style: normal;
  }

  @import "@{font-awesome}/less/core.less";
  @import "@{font-awesome}/less/larger.less";
  @import "@{font-awesome}/less/fixed-width.less";
  @import "@{font-awesome}/less/list.less";
  @import "@{font-awesome}/less/bordered-pulled.less";
  @import "@{font-awesome}/less/animated.less";
  @import "@{font-awesome}/less/rotated-flipped.less";
  @import "@{font-awesome}/less/stacked.less";
  @import "@{font-awesome}/less/icons.less";
  @import "@{font-awesome}/less/screen-reader.less";
}
